@@ -343,7 +343,7 @@ std::vector<std::string> parse_nonpositional_args(Context<E> &ctx) {
 
   // RISC-V object files contains lots of local symbols, so by default
   // we discard them. This is compatible with GNU ld.
-  if constexpr (std::is_same_v<E, RISCV64> || std::is_same_v<E, RISCV32>)
+  if constexpr (is_riscv<E>)
     ctx.arg.discard_locals = true;
 
   auto read_arg = [&](std::string name) {
@@ -985,6 +985,11 @@ std::vector<std::string> parse_nonpositional_args(Context<E> &ctx) {
     } else if (args[0] == "-z" && args.size() >= 2) {
       Warn(ctx) << "unknown command line option: -z " << args[1];
       args = args.subspan(2);
+    } else if (args[0] == "-dynamic") {
+      Fatal(ctx) << "unknown command line option: -dynamic;"
+                 << " -dynamic is a macOS linker's option. If you are trying"
+                 << " to build a binary for an Apple platform, you need to use"
+                 << " ld64.mold instead of mold or ld.mold.";
     } else {
       if (args[0][0] == '-')
         Fatal(ctx) << "unknown command line option: " << args[0];
