@@ -761,7 +761,6 @@ struct ARM64 {
   static constexpr u32 cpusubtype = CPU_SUBTYPE_ARM64_ALL;
   static constexpr u32 page_size = 16384;
   static constexpr u32 abs_rel = ARM64_RELOC_UNSIGNED;
-  static constexpr u32 word_size = 8;
   static constexpr u32 stub_size = 12;
   static constexpr u32 stub_helper_hdr_size = 24;
   static constexpr u32 stub_helper_size = 12;
@@ -772,10 +771,11 @@ struct X86_64 {
   static constexpr u32 cpusubtype = CPU_SUBTYPE_X86_64_ALL;
   static constexpr u32 page_size = 4096;
   static constexpr u32 abs_rel = X86_64_RELOC_UNSIGNED;
-  static constexpr u32 word_size = 8;
   static constexpr u32 stub_size = 6;
   static constexpr u32 stub_helper_hdr_size = 16;
   static constexpr u32 stub_helper_size = 10;
 };
 
+inline constexpr size_t word_size = 8;
+
 } // namespace mold::macho
