@@ -9,8 +9,7 @@ OBJDUMP="${OBJDUMP:-objdump}"
 MACHINE="${MACHINE:-$(uname -m)}"
 testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
-cd "$(dirname "$0")"/../..
-t=out/test/elf/$testname
+t=out/test/elf/$MACHINE/$testname
 mkdir -p $t
 
 cat <<EOF | $CC -o $t/a.o -c -xc -
@@ -33,11 +32,18 @@ cat <<EOF > $t/dyn
 { foo; extern "C++" { "baz(int)"; }; };
 EOF
 
-$CC -B. -o $t/exe $t/a.o $t/b.o -Wl,-dynamic-list=$t/dyn
+$CC -B. -o $t/exe1 $t/a.o $t/b.o -Wl,-dynamic-list=$t/dyn
 
-readelf --dyn-syms $t/exe > $t/log
-grep -q ' foo$' $t/log
-! grep -q ' bar$' $t/log || false
-grep -q ' _Z3bazi$' $t/log
+readelf --dyn-syms $t/exe1 > $t/log1
+grep -q ' foo$' $t/log1
+! grep -q ' bar$' $t/log1 || false
+grep -q ' _Z3bazi$' $t/log1
+
+$CC -B. -o $t/exe2 $t/a.o $t/b.o -Wl,--export-dynamic-symbol-list=$t/dyn
+
+readelf --dyn-syms $t/exe2 > $t/log2
+grep -q ' foo$' $t/log2
+! grep -q ' bar$' $t/log2 || false
+grep -q ' _Z3bazi$' $t/log2
 
 echo OK
