@@ -9,8 +9,7 @@ OBJDUMP="${OBJDUMP:-objdump}"
 MACHINE="${MACHINE:-$(uname -m)}"
 testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
-cd "$(dirname "$0")"/../..
-t=out/test/elf/$testname
+t=out/test/elf/$MACHINE/$testname
 mkdir -p $t
 
 [ $MACHINE = x86_64 ] || { echo skipped; exit; }
@@ -25,6 +24,6 @@ EOF
 $CC -B. -o $t/exe $t/a.o -Wl,-emit-relocs
 $QEMU $t/exe | grep -q 'Hello world'
 
-readelf -S $t/exe | grep -Pq 'rela?\.text'
+readelf -S $t/exe | grep -Eq 'rela?\.text'
 
 echo OK
