@@ -9,13 +9,12 @@ OBJDUMP="${OBJDUMP:-objdump}"
 MACHINE="${MACHINE:-$(uname -m)}"
 testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
-cd "$(dirname "$0")"/../..
-t=out/test/elf/$testname
+t=out/test/elf/$MACHINE/$testname
 mkdir -p $t
 
 rm -f $t/a.o
 touch $t/a.o
 ! $CC -B. -o $t/exe $t/a.o &> $t/log || false
-grep -q 'unknown file type: EMPTY' $t/log
+grep -q 'unknown file type' $t/log
 
 echo OK
