@@ -9,8 +9,7 @@ OBJDUMP="${OBJDUMP:-objdump}"
 MACHINE="${MACHINE:-$(uname -m)}"
 testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
-cd "$(dirname "$0")"/../..
-t=out/test/elf/$testname
+t=out/test/elf/$MACHINE/$testname
 mkdir -p $t
 
 cat <<EOF | $CC -o $t/a.o -c -x assembler -
@@ -31,7 +30,7 @@ $CC -shared -fPIC -o $t/b.so -xc /dev/null
 ./mold -o $t/exe $t/a.o $t/b.so --export-dynamic
 
 readelf --dyn-syms $t/exe > $t/log
-grep -Pq 'NOTYPE  GLOBAL DEFAULT    \d+ bar' $t/log
-grep -Pq 'NOTYPE  GLOBAL DEFAULT    \d+ _start' $t/log
+grep -Eq 'NOTYPE  GLOBAL DEFAULT    [0-9]+ bar' $t/log
+grep -Eq 'NOTYPE  GLOBAL DEFAULT    [0-9]+ _start' $t/log
 
 echo OK
