@@ -13,8 +13,7 @@ t=out/test/elf/$MACHINE/$testname
 mkdir -p $t
 
 [ $MACHINE = $(uname -m) ] || { echo skipped; exit; }
-
-[ $MACHINE = riscv64 ] && { echo skipped; exit; }
+[ $MACHINE = riscv64 -o $MACHINE = riscv32 ] && { echo skipped; exit; }
 
 which gdb >& /dev/null || { echo skipped; exit; }
 
