@@ -24,6 +24,6 @@ cat <<EOF > $t/b.script
 INPUT(-lfoo)
 EOF
 
-$CC -o $t/exe -L$t/foo/bar $t/b.script
+$CC -B. -o $t/exe -L$t/foo/bar $t/b.script
 
 echo OK
