@@ -9,8 +9,7 @@ OBJDUMP="${OBJDUMP:-objdump}"
 MACHINE="${MACHINE:-$(uname -m)}"
 testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
-cd "$(dirname "$0")"/../..
-t=out/test/elf/$testname
+t=out/test/elf/$MACHINE/$testname
 mkdir -p $t
 
 cat <<EOF | $CC -o $t/a.o -c -xc - -fno-PIE
@@ -18,6 +17,6 @@ void _start() {}
 EOF
 
 ./mold -o $t/exe $t/a.o --oformat=binary -Ttext=0x4000 -Map=$t/map
-grep -Pq '^\s+0x4000\s+\d+\s+\d+\s+\.text$' $t/map
+grep -Eq '^\s+0x4000\s+[0-9]+\s+[0-9]+\s+\.text$' $t/map
 
 echo OK
