@@ -9,14 +9,13 @@ OBJDUMP="${OBJDUMP:-objdump}"
 MACHINE="${MACHINE:-$(uname -m)}"
 testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
-cd "$(dirname "$0")"/../..
-t=out/test/elf/$testname
+t=out/test/elf/$MACHINE/$testname
 mkdir -p $t
 
 [ "$CC" = cc ] || { echo skipped; exit; }
 
 # ASAN doesn't work with LD_PRELOAD
-nm mold-wrapper.so | grep -Pq '__[at]san_init' && { echo skipped; exit; }
+nm mold-wrapper.so | grep -q '__[at]san_init' && { echo skipped; exit; }
 
 which clang >& /dev/null || { echo skipped; exit; }
 
