@@ -9,13 +9,11 @@ OBJDUMP="${OBJDUMP:-objdump}"
 MACHINE="${MACHINE:-$(uname -m)}"
 testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
-cd "$(dirname "$0")"/../..
-t=out/test/elf/$testname
+t=out/test/elf/$MACHINE/$testname
 mkdir -p $t
 
 # We need to implement R_386_GOT32X relaxation to support PIE on i386
-[ $MACHINE = i386 ] && { echo skipped; exit; }
-[ $MACHINE = i686 ] && { echo skipped; exit; }
+[ $MACHINE = i386 -o $MACHINE = i686 ] && { echo skipped; exit; }
 
 [ $MACHINE = aarch64 ] && { echo skipped; exit; }
 
