@@ -9,8 +9,7 @@ OBJDUMP="${OBJDUMP:-objdump}"
 MACHINE="${MACHINE:-$(uname -m)}"
 testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
-cd "$(dirname "$0")"/../..
-t=out/test/elf/$testname
+t=out/test/elf/$MACHINE/$testname
 mkdir -p $t
 
 cat <<EOF | $CC -o $t/a.o -c -x assembler -
@@ -30,7 +29,7 @@ this_is_global:
 module_local:
 EOF
 
-echo '{ local: module_local; };' > $t/c.map
+echo '{ local: module_local; global: *; };' > $t/c.map
 
 ./mold -o $t/exe $t/a.o $t/b.o --version-script=$t/c.map
 
@@ -38,9 +37,9 @@ readelf --symbols $t/exe > $t/log
 
 grep -Eq '0 NOTYPE  LOCAL  DEFAULT .* local1' $t/log
 grep -Eq '0 NOTYPE  LOCAL  DEFAULT .* local2' $t/log
+grep -Eq '0 NOTYPE  LOCAL  DEFAULT .* module_local' $t/log
 grep -Eq '0 NOTYPE  GLOBAL DEFAULT .* foo' $t/log
 grep -Eq '0 NOTYPE  GLOBAL DEFAULT .* bar' $t/log
 grep -Eq '0 NOTYPE  GLOBAL DEFAULT .* this_is_global' $t/log
-grep -Eq '0 NOTYPE  GLOBAL DEFAULT .* module_local' $t/log
 
 echo OK
