@@ -13,15 +13,15 @@ t=out/test/elf/$MACHINE/$testname
 mkdir -p $t
 
 if [ $MACHINE = x86_64 ]; then
-  dialect=gnu
+  mtls=-mtls-dialect=gnu
 elif [ $MACHINE = aarch64 ]; then
-  dialect=trad
-else
+  mtls=-mtls-dialect=trad
+elif [ $MACHINE '!=' riscv64 -a $MACHINE '!=' riscv32 ]; then
   echo skipped
   exit
 fi
 
-cat <<EOF | $GCC -ftls-model=local-dynamic -mtls-dialect=$dialect -fPIC -c -o $t/a.o -xc -
+cat <<EOF | $GCC -ftls-model=local-dynamic $mtls -fPIC -c -o $t/a.o -xc -
 #include <stdio.h>
 
 extern _Thread_local int foo;
@@ -38,7 +38,7 @@ int main() {
 }
 EOF
 
-cat <<EOF | $GCC -ftls-model=local-dynamic -mtls-dialect=$dialect  -fPIC -c -o $t/b.o -xc -
+cat <<EOF | $GCC -ftls-model=local-dynamic $mtls -fPIC -c -o $t/b.o -xc -
 _Thread_local int foo = 3;
 EOF
 
