@@ -13,15 +13,15 @@ t=out/test/elf/$MACHINE/$testname
 mkdir -p $t
 
 if [ $MACHINE = x86_64 ]; then
-  dialect=gnu
+  mtls=-mtls-dialect=gnu
 elif [ $MACHINE = aarch64 ]; then
-  dialect=trad
-else
+  mtls=-mtls-dialect=trad
+elif [ $MACHINE '!=' riscv64 -a $MACHINE '!=' riscv32 ]; then
   echo skipped
   exit
 fi
 
-cat <<EOF | $GCC -mtls-dialect=$dialect -c -o $t/a.o -xc -
+cat <<EOF | $GCC $mtls -c -o $t/a.o -xc -
 #include <stdio.h>
 
 extern _Thread_local int foo;
