@@ -1,18 +1,12 @@
 #!/bin/bash
 export LC_ALL=C
 set -e
-CC="${TEST_CC:-cc}"
-CXX="${TEST_CXX:-c++}"
-GCC="${TEST_GCC:-gcc}"
-GXX="${TEST_GXX:-g++}"
-OBJDUMP="${OBJDUMP:-objdump}"
-MACHINE="${MACHINE:-$(uname -m)}"
 testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
-t=out/test/macho/$MACHINE/$testname
+t=out/test/macho/$(uname -m)/$testname
 mkdir -p $t
 
-cat <<EOF | $CC -c -o $t/a.o -xc -
+cat <<EOF | cc -c -o $t/a.o -xc -
 #include <stdio.h>
 
 int main() {
@@ -20,11 +14,11 @@ int main() {
 }
 EOF
 
-clang --ld-path=./ld64 -B. -o $t/exe1 $t/a.o -Wl,-adhoc_codesign
+cc --ld-path=./ld64 -B. -o $t/exe1 $t/a.o -Wl,-adhoc_codesign
 otool -l $t/exe1 | grep -q LC_CODE_SIGNATURE
 $t/exe1 | fgrep -q 'Hello world'
 
-clang --ld-path=./ld64 -B. -o $t/exe2 $t/a.o -Wl,-no_adhoc_codesign
+cc --ld-path=./ld64 -B. -o $t/exe2 $t/a.o -Wl,-no_adhoc_codesign
 otool -l $t/exe2 > $t/log2
 ! grep -q LC_CODE_SIGNATURE $t/log2 || false
 grep -q LC_UUID $t/log2
