@@ -1,22 +1,16 @@
 #!/bin/bash
 export LC_ALL=C
 set -e
-CC="${TEST_CC:-cc}"
-CXX="${TEST_CXX:-c++}"
-GCC="${TEST_GCC:-gcc}"
-GXX="${TEST_GXX:-g++}"
-OBJDUMP="${OBJDUMP:-objdump}"
-MACHINE="${MACHINE:-$(uname -m)}"
 testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
-t=out/test/macho/$MACHINE/$testname
+t=out/test/macho/$(uname -m)/$testname
 mkdir -p $t
 
-cat <<EOF | $CC -o $t/a.o -c -xc -
+cat <<EOF | cc -o $t/a.o -c -xc -
 const char *x = "Hello world\n";
 EOF
 
-cat <<EOF | $CC -o $t/b.o -c -xc -
+cat <<EOF | cc -o $t/b.o -c -xc -
 #include <stdio.h>
 
 extern const char *x;
@@ -28,7 +22,7 @@ int main() {
 }
 EOF
 
-clang --ld-path=./ld64 -o $t/exe $t/a.o $t/b.o
+cc --ld-path=./ld64 -o $t/exe $t/a.o $t/b.o
 $t/exe | grep -q '^1 0$'
 
 echo OK
