@@ -1,18 +1,12 @@
 #!/bin/bash
 export LC_ALL=C
 set -e
-CC="${TEST_CC:-cc}"
-CXX="${TEST_CXX:-c++}"
-GCC="${TEST_GCC:-gcc}"
-GXX="${TEST_GXX:-g++}"
-OBJDUMP="${OBJDUMP:-objdump}"
-MACHINE="${MACHINE:-$(uname -m)}"
 testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
-t=out/test/macho/$MACHINE/$testname
+t=out/test/macho/$(uname -m)/$testname
 mkdir -p $t
 
-cat <<EOF | $CC -o $t/a.o -c -xc -
+cat <<EOF | cc -o $t/a.o -c -xc -
 #include <stdio.h>
 
 char msg1[] = "Hello world";
@@ -31,7 +25,7 @@ int main() {
 }
 EOF
 
-clang --ld-path=./ld64 -o $t/exe $t/a.o -Wl,-dead_strip
+cc --ld-path=./ld64 -o $t/exe $t/a.o -Wl,-dead_strip
 $t/exe | grep -q 'Hello world'
 otool -tVj $t/exe > $t/log
 grep -q 'hello:' $t/log
