@@ -1,18 +1,12 @@
 #!/bin/bash
 export LC_ALL=C
 set -e
-CC="${TEST_CC:-cc}"
-CXX="${TEST_CXX:-c++}"
-GCC="${TEST_GCC:-gcc}"
-GXX="${TEST_GXX:-g++}"
-OBJDUMP="${OBJDUMP:-objdump}"
-MACHINE="${MACHINE:-$(uname -m)}"
 testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
-t=out/test/macho/$MACHINE/$testname
+t=out/test/macho/$(uname -m)/$testname
 mkdir -p $t
 
-cat <<EOF | clang -c -o $t/a.o -xc -
+cat <<EOF | cc -c -o $t/a.o -xc -
 #include <stdio.h>
 #include <dlfcn.h>
 
@@ -30,7 +24,7 @@ int main(int argc, char **argv) {
 }
 EOF
 
-clang --ld-path=./ld64 -o $t/exe $t/a.o
+cc --ld-path=./ld64 -o $t/exe $t/a.o
 $t/exe | grep -q sname=main
 
 echo OK
