@@ -1,18 +1,12 @@
 #!/bin/bash
 export LC_ALL=C
 set -e
-CC="${TEST_CC:-cc}"
-CXX="${TEST_CXX:-c++}"
-GCC="${TEST_GCC:-gcc}"
-GXX="${TEST_GXX:-g++}"
-OBJDUMP="${OBJDUMP:-objdump}"
-MACHINE="${MACHINE:-$(uname -m)}"
 testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
-t=out/test/macho/$MACHINE/$testname
+t=out/test/macho/$(uname -m)/$testname
 mkdir -p $t
 
-cat <<EOF | $CC -o $t/a.o -c -xc -
+cat <<EOF | cc -o $t/a.o -c -xc -
 #include <stdio.h>
 
 int main();
@@ -36,10 +30,10 @@ _main
 _print
 EOF
 
-clang --ld-path=./ld64 -o $t/exe1 $t/a.o -Wl,-order_file,$t/order1
+cc --ld-path=./ld64 -o $t/exe1 $t/a.o -Wl,-order_file,$t/order1
 $t/exe1 | grep -q '^1$'
 
-clang --ld-path=./ld64 -o $t/exe2 $t/a.o -Wl,-order_file,$t/order2
+cc --ld-path=./ld64 -o $t/exe2 $t/a.o -Wl,-order_file,$t/order2
 $t/exe2 | grep -q '^0$'
 
 echo OK
