@@ -1,32 +1,26 @@
 #!/bin/bash
 export LC_ALL=C
 set -e
-CC="${TEST_CC:-cc}"
-CXX="${TEST_CXX:-c++}"
-GCC="${TEST_GCC:-gcc}"
-GXX="${TEST_GXX:-g++}"
-OBJDUMP="${OBJDUMP:-objdump}"
-MACHINE="${MACHINE:-$(uname -m)}"
 testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
-t=out/test/macho/$MACHINE/$testname
+t=out/test/macho/$(uname -m)/$testname
 mkdir -p $t
 
 [ "`uname -p`" = arm ] && { echo skipped; exit; }
 
-cat <<EOF | $CC -o $t/a.o -c -xc -
+cat <<EOF | cc -o $t/a.o -c -xc -
 #include <stdio.h>
 int main() {
   printf("Hello world\n");
 }
 EOF
 
-clang --ld-path=./ld64 -o $t/exe $t/a.o
+cc --ld-path=./ld64 -o $t/exe $t/a.o
 
 otool -l $t/exe | grep -A5 'segname __PAGEZERO' | \
   grep -q 'vmsize 0x0000000100000000'
 
-clang --ld-path=./ld64 -o $t/exe $t/a.o -Wl,-pagezero_size,0x10000
+cc --ld-path=./ld64 -o $t/exe $t/a.o -Wl,-pagezero_size,0x10000
 $t/exe | grep -q 'Hello world'
 
 otool -l $t/exe | grep -A5 'segname __PAGEZERO' | \
