@@ -1,18 +1,12 @@
 #!/bin/bash
 export LC_ALL=C
 set -e
-CC="${TEST_CC:-cc}"
-CXX="${TEST_CXX:-c++}"
-GCC="${TEST_GCC:-gcc}"
-GXX="${TEST_GXX:-g++}"
-OBJDUMP="${OBJDUMP:-objdump}"
-MACHINE="${MACHINE:-$(uname -m)}"
 testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
 t=out/test/macho/$testname
 mkdir -p $t
 
-cat <<'EOF' | $CC -o $t/a.o -c -xc -
+cat <<'EOF' | cc -o $t/a.o -c -xc -
 #include <stdio.h>
 #include <stdint.h>
 
@@ -33,7 +27,7 @@ int main() {
 }
 EOF
 
-clang --ld-path=./ld64 -o $t/exe $t/a.o
+cc --ld-path=./ld64 -o $t/exe $t/a.o
 $t/exe > /dev/null
 
 echo OK
