@@ -1,18 +1,12 @@
 #!/bin/bash
 export LC_ALL=C
 set -e
-CC="${TEST_CC:-cc}"
-CXX="${TEST_CXX:-c++}"
-GCC="${TEST_GCC:-gcc}"
-GXX="${TEST_GXX:-g++}"
-OBJDUMP="${OBJDUMP:-objdump}"
-MACHINE="${MACHINE:-$(uname -m)}"
 testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
-t=out/test/macho/$MACHINE/$testname
+t=out/test/macho/$(uname -m)/$testname
 mkdir -p $t
 
-cat <<EOF | $CC -o $t/a.o -c -xassembler -
+cat <<EOF | cc -o $t/a.o -c -xassembler -
 .subsections_via_symbols
 .globl _fn1, _fn2
 .text
@@ -23,7 +17,7 @@ _fn2:
   nop
 EOF
 
-cat <<EOF | $CC -o $t/b.o -c -xassembler -
+cat <<EOF | cc -o $t/b.o -c -xassembler -
 .globl _fn3, _fn4
 .text
 .align 16
@@ -33,7 +27,7 @@ _fn4:
   .byte 0xcc
 EOF
 
-cat <<EOF | $CC -o $t/c.o -c -xc -
+cat <<EOF | cc -o $t/c.o -c -xc -
 #include <stdio.h>
 
 void fn1();
@@ -46,7 +40,7 @@ int main() {
 }
 EOF
 
-clang --ld-path=./ld64 -o $t/exe $t/a.o $t/b.o $t/c.o
+cc --ld-path=./ld64 -o $t/exe $t/a.o $t/b.o $t/c.o
 $t/exe | grep -q '^16 1$'
 
 echo OK
