@@ -1,18 +1,12 @@
 #!/bin/bash
 export LC_ALL=C
 set -e
-CC="${TEST_CC:-cc}"
-CXX="${TEST_CXX:-c++}"
-GCC="${TEST_GCC:-gcc}"
-GXX="${TEST_GXX:-g++}"
-OBJDUMP="${OBJDUMP:-objdump}"
-MACHINE="${MACHINE:-$(uname -m)}"
 testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
-t=out/test/macho/$MACHINE/$testname
+t=out/test/macho/$(uname -m)/$testname
 mkdir -p $t
 
-cat <<EOF | $CC -o $t/a.o -c -xc -
+cat <<EOF | cc -o $t/a.o -c -xc -
 #include <stdio.h>
 void hello() {
   printf("Hello world\n");
@@ -21,14 +15,14 @@ EOF
 
 lipo $t/a.o -create -output $t/fat.o
 
-cat <<EOF | $CC -o $t/b.o -c -xc -
+cat <<EOF | cc -o $t/b.o -c -xc -
 void hello();
 int main() {
   hello();
 }
 EOF
 
-clang --ld-path=./ld64 -o $t/exe $t/a.o $t/b.o
+cc --ld-path=./ld64 -o $t/exe $t/a.o $t/b.o
 $t/exe | grep -q 'Hello world'
 
 echo OK
