@@ -1,18 +1,12 @@
 #!/bin/bash
 export LC_ALL=C
 set -e
-CC="${TEST_CC:-cc}"
-CXX="${TEST_CXX:-c++}"
-GCC="${TEST_GCC:-gcc}"
-GXX="${TEST_GXX:-g++}"
-OBJDUMP="${OBJDUMP:-objdump}"
-MACHINE="${MACHINE:-$(uname -m)}"
 testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
-t=out/test/macho/$MACHINE/$testname
+t=out/test/macho/$(uname -m)/$testname
 mkdir -p $t
 
-cat <<EOF | clang -o $t/a.o -c -xc -
+cat <<EOF | cc -o $t/a.o -c -xc -
 int foo();
 int main() { foo(); }
 EOF
@@ -37,6 +31,6 @@ exports:
 ...
 EOF
 
-clang --ld-path=./ld64 -o $t/exe $t/a.o $t/b.tbd
+cc --ld-path=./ld64 -o $t/exe $t/a.o $t/b.tbd
 
 echo OK
